package ileri;

import static ileri.Devices.firstNonLoopbackAddress;
import net.sbbi.upnp.impls.InternetGatewayDevice;

/**
 * A routing device that can be used for <i>UPnP</i> mapping.
 * 
 * @author Thomas G. P. Nappo
 */
public class Device {

	private final InternetGatewayDevice backing;

	public Device(InternetGatewayDevice backing) {
		this.backing = backing;
	}

	protected InternetGatewayDevice getBacking() {
		return backing;
	}

	/**
	 * Adds a <i>UPnP</i> mapping.
	 * 
	 * @param identifier
	 *            The identifier of the mapping.
	 * @param port
	 *            The port of the mapping.
	 * @param protocol
	 *            The protocol of the mapping.
	 * @return Whether or not the mapping was successful.
	 */
	public boolean map(String identifier, int port, Protocol protocol) {
		try {
			unmap(port, protocol); // override existing mappings
			getBacking().addPortMapping(identifier, null, port, port,
					firstNonLoopbackAddress().getHostAddress(), 0,
					protocol.name());
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Adds a <i>UPnP</i> mapping.
	 * 
	 * @param port
	 *            The port of the mapping.
	 * @param protocol
	 *            The protocol of the mapping.
	 * @return Whether or not the mapping was successful.
	 */
	public boolean map(int port, Protocol protocol) {
		return map("Nexu UPNP (port=" + port + ", protocol=" + protocol.name()
				+ ")", port, protocol);
	}

	/**
	 * Removes a <i>UPnP</i> mapping.
	 * 
	 * @param port
	 *            The port of the mapping.
	 * @param protocol
	 *            The protocol of the mapping.
	 * @return Whether or not the unmapping was successful.
	 */
	public boolean unmap(int port, Protocol protocol) {
		try {
			getBacking().deletePortMapping(null, port, protocol.name());
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	public String getName() {
		return backing.getIGDRootDevice().getFriendlyName();
	}

	public String getModel() {
		return backing.getIGDRootDevice().getModelName();
	}

	public String getManufacturer() {
		return backing.getIGDRootDevice().getManufacturer();
	}

	public String getDescription() {
		return backing.getIGDRootDevice().getModelDescription();
	}

	/**
	 * Gets the external address of the device.
	 * 
	 * @return {@code null} if the external address could not be found.
	 */
	public String getExternalAddress() {
		try {
			return backing.getExternalIPAddress();
		} catch (Exception e) {
			return null;
		}
	}

}