package ileri;

import static java.net.NetworkInterface.getNetworkInterfaces;

import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import net.sbbi.upnp.impls.InternetGatewayDevice;

/**
 * Utility methods for dealing with <i>UPnP</i> devices.
 * 
 * @author Thomas G. P. Nappo
 */
public final class Devices {

	/**
	 * Searches for <i>UPnP</i> devices for an amount of time.
	 * 
	 * @param timeout
	 *            The amount of time to search for devices (in milliseconds)
	 * @return The discovered devices.
	 */
	public static List<Device> search(int timeout) {
		List<Device> devices = new ArrayList<Device>();
		try {
			InternetGatewayDevice[] backings = InternetGatewayDevice
					.getDevices(timeout);
			for (InternetGatewayDevice backing : backings) {
				devices.add(new Device(backing));
			}
		} catch (IOException e) {
		}
		return devices;
	}

	/**
	 * Searches for <i>UPnP</i> devices for <tt>1 second</tt>.
	 * 
	 * @return The discovered devices.
	 */
	public static List<Device> search() {
		return search(1000);
	}

	/**
	 * Searches for the first (fastest) <i>UPnP</i> device.
	 * 
	 * <p>
	 * This method is blocking; it will continually search for a device until
	 * one if found.
	 * </p>
	 * 
	 * @param timeout
	 *            The amount of time to search for devices in each discovery
	 *            search.
	 * @return The first (fastest) <i>UPnP</i> device that was discovered.
	 */
	public static Device fastest(int timeout) {
		Device device = null;
		do {
			List<Device> devices = search(timeout);
			if (!devices.isEmpty()) {
				device = devices.get(0);
			}
		} while (device == null);
		return device;
	}

	/**
	 * Searches for the first (fastest) <i>UPnP</i> device giving
	 * <tt>100 milliseconds</tt> for each discovery search.
	 * 
	 * <p>
	 * This method is blocking; it will continually search for a device until
	 * one if found.
	 * </p>
	 * 
	 * @return The first (fastest) <i>UPnP</i> device that was discovered.
	 */
	public static Device fastest() {
		return fastest(100);
	}

	private static InetAddress firstNonLoopbackAddress;

	public static InetAddress firstNonLoopbackAddress() throws SocketException {
		if (firstNonLoopbackAddress != null) {
			return firstNonLoopbackAddress;
		}

		Enumeration<NetworkInterface> nis = getNetworkInterfaces();
		while (nis.hasMoreElements()) {
			NetworkInterface ni = (NetworkInterface) nis.nextElement();
			for (Enumeration<InetAddress> ii = ni.getInetAddresses(); ii
					.hasMoreElements();) {
				InetAddress address = (InetAddress) ii.nextElement();
				if (!address.isLoopbackAddress()) {
					firstNonLoopbackAddress = address;
					return address;
				}
			}
		}

		return null;
	}

}